@@ -192,6 +192,7 @@ services:
 
 	_, err := ParseCompose([]composetypes.ConfigFile{{Content: dt}}, nil)
 	require.Error(t, err)
+	require.ErrorContains(t, err, `has neither an image nor a build context specified`)
 }
 
 func TestAdvancedNetwork(t *testing.T) {
@@ -620,7 +621,7 @@ services:
   foo:
 `),
 			isCompose: true,
-			wantErr:   true,
+			wantErr:   false,
 		},
 		{
 			name: "build",
@@ -666,6 +667,52 @@ target "default" {
 			isCompose: false,
 			wantErr:   false,
 		},
+		{
+			name: "json",
+			fn:   "docker-bake.json",
+			dt: []byte(`
+{
+  "group": [
+    {
+      "targets": [
+        "my-service"
+      ]
+    }
+  ],
+  "target": [
+    {
+      "context": ".",
+      "dockerfile": "Dockerfile"
+    }
+  ]
+}
+`),
+			isCompose: false,
+			wantErr:   false,
+		},
+		{
+			name: "json unknown ext",
+			fn:   "docker-bake.foo",
+			dt: []byte(`
+{
+  "group": [
+    {
+      "targets": [
+        "my-service"
+      ]
+    }
+  ],
+  "target": [
+    {
+      "context": ".",
+      "dockerfile": "Dockerfile"
+    }
+  ]
+}
+`),
+			isCompose: false,
+			wantErr:   true,
+		},
 	}
 	for _, tt := range cases {
 		t.Run(tt.name, func(t *testing.T) {
@@ -846,6 +893,90 @@ services:
 	require.Equal(t, map[string]string{"base": "target:base"}, c.Targets[1].Contexts)
 }
 
+func TestDotEnvDir(t *testing.T) {
+	tmpdir := t.TempDir()
+	require.NoError(t, os.Mkdir(filepath.Join(tmpdir, ".env"), 0755))
+
+	dt := []byte(`
+services:
+  foo:
+    build:
+     context: .
+`)
+
+	chdir(t, tmpdir)
+	_, err := ParseComposeFiles([]File{{Name: "compose.yml", Data: dt}})
+	require.NoError(t, err)
+}
+
+func TestDotEnvEvaluate(t *testing.T) {
+	tmpdir := t.TempDir()
+
+	err := os.WriteFile(filepath.Join(tmpdir, ".env"), []byte(`
+TEST_VALUE=${SYSTEM_VALUE:?system_value_not_set}
+FOO_VALUE=${TEST_VALUE:?test_value_not_set}
+`), 0644)
+	require.NoError(t, err)
+
+	dt := []byte(`
+services:
+  test:
+    build:
+      args:
+        TEST_VALUE:
+        FOO_VALUE:
+`)
+
+	t.Setenv("SYSTEM_VALUE", "abc")
+
+	chdir(t, tmpdir)
+	c, err := ParseComposeFiles([]File{{Name: "compose.yml", Data: dt}})
+	require.NoError(t, err)
+	require.Equal(t, map[string]*string{"TEST_VALUE": ptrstr("abc"), "FOO_VALUE": ptrstr("abc")}, c.Targets[0].Args)
+}
+
+func TestUnknownField(t *testing.T) {
+	tmpdir := t.TempDir()
+	dt := []byte(`
+services:
+  webapp:
+    bar: baz
+    build:
+     context: .
+
+foo:
+  - bar.baz
+`)
+
+	chdir(t, tmpdir)
+	_, err := ParseComposeFiles([]File{{Name: "compose.yml", Data: dt}})
+	require.NoError(t, err)
+}
+
+func TestUnknownBuildField(t *testing.T) {
+	tmpdir := t.TempDir()
+	dt := []byte(`
+services:
+  webapp:
+    build:
+     context: .
+     foo: bar
+`)
+
+	chdir(t, tmpdir)
+	_, err := ParseComposeFiles([]File{{Name: "compose.yml", Data: dt}})
+	require.Error(t, err)
+	require.ErrorContains(t, err, `additional properties 'foo' not allowed`)
+}
+
+func TestEmptyComposeFile(t *testing.T) {
+	tmpdir := t.TempDir()
+	chdir(t, tmpdir)
+	_, err := ParseComposeFiles([]File{{Name: "compose.yml", Data: []byte(``)}})
+	require.Error(t, err)
+	require.ErrorContains(t, err, `empty compose file`) // https://github.com/compose-spec/compose-go/blob/a42e7579d813e64c0c1f598a666358bc0c0a0eb4/loader/loader.go#L542
+}
+
 // chdir changes the current working directory to the named directory,
 // and then restore the original working directory at the end of the test.
 func chdir(t *testing.T, dir string) {
