@@ -4,7 +4,6 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/docker/buildx/builder"
 	"github.com/docker/buildx/util/cobrautil/completion"
 	"github.com/docker/buildx/util/desktop"
 	"github.com/docker/cli/cli/command"
@@ -19,21 +18,11 @@ type openOptions struct {
 }
 
 func runOpen(ctx context.Context, dockerCli command.Cli, opts openOptions) error {
-	b, err := builder.New(dockerCli, builder.WithName(opts.builder))
+	nodes, err := loadNodes(ctx, dockerCli, opts.builder)
 	if err != nil {
 		return err
 	}
 
-	nodes, err := b.LoadNodes(ctx)
-	if err != nil {
-		return err
-	}
-	for _, node := range nodes {
-		if node.Err != nil {
-			return node.Err
-		}
-	}
-
 	recs, err := queryRecords(ctx, opts.ref, nodes, nil)
 	if err != nil {
 		return err
